const fs = require('fs');
const http = require('http');

const chalk = require('chalk');

const WebSocketServer = require('websocket').server;

const port = 40255;

new Promise((res, rej) => {
        fs.readFile('./example-client.html', 'utf8', (err, exampleClient) => {
            if (err) {
                rej(err);
            } else {
                res(exampleClient);
            }
        })
    })
    .then(exampleClient => {
        const webserver = startWebServer(exampleClient);
        const webSocketServer = startWebSocketServer(webserver);
    })
    .catch((err) => {
        console.error('something went wrong while starting the server ', err);
    })


function startWebServer(exampleClient) {
    const header = {
        'Content-Type': 'text/html',
        'Content-Length': Buffer.byteLength(exampleClient)
    };

    const server = http.createServer((request, response) => {
        if (request.url == "/") {
            response.writeHead(200, header);
            response.write(exampleClient, 'utf8');
        } else {
            response.writeHead(404);
        }
        response.end();
    })

    server.listen(port);
    process.on('exit', () => server.close())

    console.log(chalk.green("[Web Server]") + " startet web server on port " + port);
    return server;
}

const pref = chalk.gray("[WebSocket Server]")
const prettyID = id => chalk.yellow("[" + (("" + id).padStart(3, '0')) + "]");

let connectionCounter = 1;

function onRequestAccepted(connection) {
    id = connectionCounter++;

    console.log(pref + " " + prettyID(id) + " " + chalk.green("connected"));

    connection.on('close', function (reasonCode, description) {
        console.log(pref + " " + prettyID(id) + " " + chalk.red("disconnected"));
    });
}

function onRequestRejected() {
    console.error(chalk.red('[WebSocket Server]') + " rejected websocket connection request");
}

function startWebSocketServer(httpServer) {

    const webSocketServer = new WebSocketServer({
        httpServer: httpServer,
        autoAcceptConnections: false
    });

    console.log(chalk.green('[WebSocket Server]') + " startet WebSocket server on port " + port);

    process.on('exit', () => webSocketServer.shutDown());

    webSocketServer.on('request', function (request) {
        request.on('requestAccepted', onRequestAccepted);
        request.on('requestRejected', onRequestRejected);

        /**
         * WARNING there is no checking in place, any origin can connect!
         */
        request.accept('simple-connect', request.origin);
    });

    return webSocketServer;
}